header {
  color: #d000ff;
  background-color: #360342;
  white-space: nowrap;
}

header a {
  color: #dcdcdc;
  font-size: 16px;
  line-height: 1;
  text-decoration: none;
  position: relative;
  display: inline-block;
  transition: color 0.2s ease, tranform 0.2s ease;
}

header a:hover {
  color: #d000ff;
  text-decoration: underline;
  transform: translateY(-1px);
}

#header-left a {
  margin-right: 18px;
}

#header-right a {
  padding-right: 0.15rem;
}

#home {
  color: #f0f0f0;
  font-size: 32px;
  font-family: "Kristi", sans-serif;
  font-weight: 400;
  font-style: normal;
}

#home:hover {
  color:#d000ff;
  text-decoration: none;
  transform: none;
}

.btn-custom {
  color: white;
  border-color: white;
  background-color: transparent;
  transition: border-color 0.2s ease, background-color 0.2s ease;
}

.btn-custom:hover {
  border-color: #9200b3;
  background-color: #9200b3;
}
